@@ -1,6 +1,10 @@
 #!/usr/bin/perl
 
 use v5.36;
+use utf8;
+
+no warnings qw( experimental::builtin experimental::for_list );
+use builtin qw( true false indexed );
 
 use Text::Treesitter;
 use Text::Treesitter::QueryCursor;
@@ -17,6 +21,7 @@ GetOptions(
    'unrecognised|u' => \( my $PRINT_UNRECOGNISED ),
    'directory|d=s'  => \( my $LANGUAGE_DIR ),
    'use-theme|U'    => \( my $USE_THEME ),
+   'folding|F'      => \( my $FOLDING ),
 ) or exit 1;
 
 STDOUT->binmode( ':encoding(UTF-8)' );
@@ -28,6 +33,13 @@ my $ts = Text::Treesitter->new(
 
 my $query = $ts->load_query_file( "queries/highlights.scm" );
 
+my $query_folding;
+
+my $folding_path;
+if( $FOLDING and -f ( $folding_path = $ts->query_file_path( "queries/fold.scm" ) ) ) {
+   $query_folding = $ts->load_query_file( $folding_path );
+}
+
 my $str = String::Tagged->new( read_text $ARGV[0] // "/dev/stdin" );
 
 my $tree = $ts->parse_string( $str );
@@ -36,6 +48,7 @@ my %FORMATS = (
    # Names stolen from tree-sitter's highlight theme
    attribute  => { fg => "vga:cyan", italic => 1 },
    comment    => { fg => "xterm:15", bg => "xterm:54", italic => 1 },
+   decorator  => { fg => "xterm:140", italic => 1 },
    function   => { fg => "xterm:147", },
    keyword    => { fg => "vga:yellow", bold => 1 },
    module     => { fg => "vga:green", bold => 1 },
@@ -86,6 +99,46 @@ foreach ( values %FORMATS ) {
 
 my $qc = Text::Treesitter::QueryCursor->new;
 
+# For ease of code management, line numbers are 0-indexed
+
+my @fold_regions;
+my $fold_deepest = 0;
+if( $FOLDING and $query_folding ) {
+   $qc->exec( $query_folding, $tree->root_node );
+
+   my @regions_applied;
+
+   while( my $match = $qc->next_match ) {
+      my @captures = $match->captures;
+
+      next unless $query_folding->test_predicates_for_match( $tree, $match, \@captures );
+
+      my @nodes;
+
+      foreach my $capture ( @captures ) {
+         next unless $query_folding->capture_name_for_id( $capture->capture_id ) eq "fold";
+
+         push @nodes, $capture->node;
+      }
+
+      # Some fold patterns capture multiple toplevel nodes.
+      my $startline = ( $nodes[ 0]->start_point )[0];
+      my $endline   = ( $nodes[-1]->end_point )[0];
+
+      die "TODO: This fold region starts earlier than the previous one"
+         if @fold_regions and $startline < $fold_regions[-1][0];
+
+      next if $startline == $endline;
+
+      push @fold_regions, [ $startline, $endline ];
+
+      pop @regions_applied while @regions_applied and $regions_applied[-1][1] < $startline;
+      push @regions_applied, $fold_regions[-1];
+
+      $fold_deepest = scalar @regions_applied if @regions_applied > $fold_deepest;
+   }
+}
+
 $qc->exec( $query, $tree->root_node );
 
 my %UNRECOGNISED_CAPTURES;
@@ -116,7 +169,20 @@ while( my $match = $qc->next_match ) {
    }
 }
 
-foreach my $line ( $str->split( qr/\n/ ) ) {
+foreach my ( $lnum, $line ) ( indexed $str->split( qr/\n/ ) ) {
+   if( $FOLDING ) {
+      my @regions = grep { $_->[0] <= $lnum and $lnum <= $_->[1] } @fold_regions;
+      my $final_here;
+      my $markers = join "", map {
+         $_->[0] == $lnum ? ( $final_here = true, "┌" )[1] :
+         $_->[1] == $lnum ? ( $final_here = true, "└" )[1] :
+                            "│";
+      } @regions;
+
+      $markers .= ( $final_here ? "─" : " " ) while length $markers < $fold_deepest;
+
+      print $markers . " ";
+   }
    String::Tagged::Terminal->new_from_formatting( $line )
       ->say_to_terminal;
 }
