@@ -6,7 +6,7 @@
 use v5.26;
 use Object::Pad 0.70 ':experimental(adjust_params)';
 
-package Text::Treesitter 0.08;
+package Text::Treesitter 0.09;
 class Text::Treesitter
    :strict(params);
 
@@ -272,6 +272,21 @@ method load_query_string ( $src )
    return Text::Treesitter::Query->new( $_lang, $src );
 }
 
+=head2 query_file_path
+
+   $path = %ts->query_file_path( $path );
+
+If a file exists of the given path, then it is returned directly. Otherwise,
+returns a path within the language directory given by C<lang_dir>.
+
+=cut
+
+method query_file_path ( $path )
+{
+   -f $path and return $path;
+   return "$_lang_dir/$path";
+}
+
 =head2 load_query_file
 
    $query = $ts->load_query_file( $path );
@@ -285,8 +300,7 @@ language directory given by C<lang_dir>. The first one found will be loaded.
 
 method load_query_file ( $path )
 {
-   -f $path or $path = "$_lang_dir/$path";
-   return $self->load_query_string( read_text $path );
+   return $self->load_query_string( read_text $self->query_file_path( $path ) );
 }
 
 =head1 TODO
