@@ -6,7 +6,7 @@
 use v5.26;
 use Object::Pad 0.70;
 
-package Text::Treesitter::Node 0.08;
+package Text::Treesitter::Node 0.09;
 class Text::Treesitter::Node
    :strict(params);
 
@@ -161,7 +161,7 @@ Returns true if the node or any of its descendents represents a syntax error.
    $parent = $node->parent;
 
 Returns the node's immediate parent; the node from which this node was
-obtained.
+obtained. Returns C<undef> on the root node.
 
 =head2 child_count
 
@@ -251,7 +251,8 @@ method end_char   { return $tree->byte_to_char( $self->end_byte ) }
 
 method parent ()
 {
-   return Text::Treesitter::Node->new( node => $node->parent, tree => $tree );
+   my $parent = $node->parent or return undef;
+   return Text::Treesitter::Node->new( node => $parent, tree => $tree );
 }
 
 method child_nodes ()
@@ -276,6 +277,51 @@ method try_child_by_field_name ( $field_name )
    return Text::Treesitter::Node->new( node => $child, tree => $tree )
 }
 
+=head2 debug_sprintf
+
+   $str = $node->debug_sprintf();
+
+Returns a debugging test string that represents the node and all its child
+nodes, in a format similar to F<tree-sitter>'s usual S-expr notation.
+
+Basic named nodes are printed with their name in parens; C<(type)>. Anonymous
+nodes have their text string in quotes; C<"text">. Child nodes of named are
+included within the parens of the type name. Field names are printed as
+prefixes with a colon.
+
+   (node)
+
+   (node (children) (go) "here")
+
+   (node left: (node) right: (node))
+
+=cut
+
+method debug_sprintf ()
+{
+   # Unnamed nodes are just their own text
+   if( !$self->is_named ) {
+      return sprintf qq("%s"), $self->text =~ s/([\\"])/\\$1/gr;
+   }
+
+   my @named_children = $self->field_names_with_child_nodes;
+
+   my $ret = "(" . $self->type;
+
+   while( @named_children ) {
+      my $fieldname = shift @named_children;
+      my $child     = shift @named_children;
+
+      $ret .= " ";
+      $ret .= "$fieldname\: " if defined $fieldname;
+      $ret .= $child->debug_sprintf;
+   }
+
+   $ret .= ")";
+
+   return $ret;
+}
+
 =head1 TODO
 
 The following C library functions are currently unhandled:
