@@ -3,7 +3,7 @@
 #
 #  (C) Paul Evans, 2023 -- leonerd@leonerd.org.uk
 
-package Text::Treesitter::Query 0.08;
+package Text::Treesitter::Query 0.09;
 
 use v5.14;
 use warnings;
@@ -14,6 +14,12 @@ use List::Util qw( any );
 
 require Text::Treesitter::_XS;
 
+use Exporter 'import';
+our @EXPORT_OK = qw(
+   TSQuantifierZero TSQuantifierZeroOrOne TSQuantifierZeroOrMore
+   TSQuantifierOne  TSQuantifierOneOrMore
+);
+
 =head1 NAME
 
 C<Text::Treesitter::Query> - represents a set of F<tree-sitter> query patterns
@@ -178,6 +184,24 @@ string values.
 
 (This predicate is inspired by F<nvim>.)
 
+=head3 has-parent?
+
+   (#has-parent? @capture type names)
+
+Accepts if the immediate parent of first argument (which must be a node
+capture) has a type that is any of the subsequent type names.
+
+(This predicate is inspired by F<nvim>.)
+
+=head3 has-ancestor?
+
+   (#has-ancestor? @capture type names)
+
+Accepts if any ancestor of the first argument (which mus be a node capture)
+has a type that is any of the subsequent type names.
+
+(This predicate is inspired by F<nvim>.)
+
 =cut
 
 # Not documented as a method but handy for unit testing
@@ -185,21 +209,41 @@ sub test_predicate ( $self, $func, @args )
 {
    my $invert = $func =~ s/^not-//;
 
+   # Convert args to text
+   my @argtext = map { ref($_) eq "Text::Treesitter::Node" ? $_->text : $_ } @args;
+
    match( $func : eq ) {
       case( "eq?" ) {
-         return $invert ^ ($args[0] eq $args[1]);
+         return $invert ^ ($argtext[0] eq $argtext[1]);
       }
       case( "match?" ) {
          # This is an unanchored match; use ^ and $ to anchor it if required
-         return $invert ^ ($args[0] =~ m/$args[1]/);
+         return $invert ^ ($argtext[0] =~ m/$argtext[1]/);
       }
       case( "contains?" ) {
-         my $str = shift @args;
-         return $invert ^ any { index( $str, $_ ) > -1 } @args;
+         my $str = shift @argtext;
+         return $invert ^ any { index( $str, $_ ) > -1 } @argtext;
       }
       case( "any-of?" ) {
-         my $str = shift @args;
-         return $invert ^ any { $str eq $_ } @args;
+         my $str = shift @argtext;
+         return $invert ^ any { $str eq $_ } @argtext;
+      }
+      case( "has-parent?" ) {
+         my $node = $args[0]->parent; shift @argtext;
+         $node or return $invert;
+         $node->is_named or return $invert;
+         my $type = $node->type;
+         return $invert ^ any { $type eq $_ } @argtext;
+      }
+      case( "has-ancestor?" ) {
+         my $node = $args[0]->parent; shift @argtext;
+         while( $node ) {
+            $node->is_named or next;
+            my $type = $node->type;
+            return !$invert if any { $type eq $_ } @argtext;
+            $node = $node->parent;
+         }
+         return $invert;
       }
       default {
          warn "Unrecognised predicate test '$func'";
@@ -220,12 +264,7 @@ sub test_predicates_for_match ( $self, $tree, $match, $_captures = undef )
       my $id   = $capture->capture_id;
       my $node = $capture->node;
 
-      my $start = $tree->byte_to_char( $node->start_byte );
-      my $end   = $tree->byte_to_char( $node->end_byte );
-
-      my $text = $tree->text_substring( $start, $end );
-
-      $captures_by_id{ $capture->capture_id } = $text;
+      $captures_by_id{ $id } = $node;
    }
 
    foreach my $predicate ( @predicates ) {
@@ -238,6 +277,23 @@ sub test_predicates_for_match ( $self, $tree, $match, $_captures = undef )
    return 1;
 }
 
+=head2 capture_quantifier_for_id
+
+   $quant = $query->capture_quantifier_for_id( $pattern_id, $capture_id );
+
+I<Since version 0.09.>
+
+Returns the quantifier associated with the given capture of the given pattern.
+This will match one of the following C<TSQuantifier*> constants
+
+   TSQuantifierZero
+   TSQuantifierZeroOrOne
+   TSQuantifierZeroOrMore
+   TSQuantifierOne
+   TSQuantifierOneOrMore
+
+=cut
+
 =head1 TODO
 
 The following C library functions are currently unhandled:
@@ -245,7 +301,6 @@ The following C library functions are currently unhandled:
    ts_query_start_byte_for_pattern
    ts_query_is_pattern_rooted
    ts_query_is_pattern_guaranteed_at_step
-   ts_query_capture_quantifier_for_id
    ts_query_disable_capture
    ts_query_disable_pattern
 
